""" Signatures for PDELab local opreator assembly functions """

from dune.perftool import Restriction
from dune.perftool.generation import symbol
from dune.perftool.pdelab.geometry import (name_geometry_wrapper,
                                           type_geometry_wrapper,
                                           )
from dune.perftool.pdelab.argument import (name_accumulation_variable,
                                           type_accumulation_variable,
                                           name_coefficientcontainer,
                                           type_coefficientcontainer,
                                           name_applycontainer,
                                           name_testfunctionspace,
                                           type_testfunctionspace,
                                           name_trialfunctionspace,
                                           type_trialfunctionspace,
                                           )


@symbol
def alpha_volume_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu = name_trialfunctionspace(Restriction.NONE)
    lfsvt = type_testfunctionspace()
    lfsv = name_testfunctionspace(Restriction.NONE)
    cct = type_coefficientcontainer()
    cc = name_coefficientcontainer(Restriction.NONE)
    avt = type_accumulation_variable()
    av = name_accumulation_variable((Restriction.NONE,))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void alpha_volume(const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}) const'.format(geot,
                                                                                                             geo,
                                                                                                             lfsut,
                                                                                                             lfsu,
                                                                                                             cct,
                                                                                                             cc,
                                                                                                             lfsvt,
                                                                                                             lfsv,
                                                                                                             avt,
                                                                                                             av,
                                                                                                             )
            ]


@symbol
def alpha_boundary_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu = name_trialfunctionspace(Restriction.NEGATIVE)
    lfsvt = type_testfunctionspace()
    lfsv = name_testfunctionspace(Restriction.NEGATIVE)
    cct = type_coefficientcontainer()
    cc = name_coefficientcontainer(Restriction.NEGATIVE)
    avt = type_accumulation_variable()
    av = name_accumulation_variable((Restriction.NEGATIVE,))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void alpha_boundary(const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}) const'.format(geot,
                                                                                                               geo,
                                                                                                               lfsut,
                                                                                                               lfsu,
                                                                                                               cct,
                                                                                                               cc,
                                                                                                               lfsvt,
                                                                                                               lfsv,
                                                                                                               avt,
                                                                                                               av,
                                                                                                               )
            ]


@symbol
def alpha_skeleton_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu_s = name_trialfunctionspace(Restriction.NEGATIVE)
    lfsu_n = name_trialfunctionspace(Restriction.POSITIVE)
    lfsvt = type_testfunctionspace()
    lfsv_s = name_testfunctionspace(Restriction.NEGATIVE)
    lfsv_n = name_testfunctionspace(Restriction.POSITIVE)
    cct = type_coefficientcontainer()
    cc_s = name_coefficientcontainer(Restriction.NEGATIVE)
    cc_n = name_coefficientcontainer(Restriction.POSITIVE)
    avt = type_accumulation_variable()
    av_s = name_accumulation_variable((Restriction.NEGATIVE,))
    av_n = name_accumulation_variable((Restriction.POSITIVE,))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void alpha_skeleton(const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}, {}& {}) const'.format(geot,
                                                                                                                                                                 geo,
                                                                                                                                                                 lfsut,
                                                                                                                                                                 lfsu_s,
                                                                                                                                                                 cct,
                                                                                                                                                                 cc_s,
                                                                                                                                                                 lfsvt,
                                                                                                                                                                 lfsv_s,
                                                                                                                                                                 lfsut,
                                                                                                                                                                 lfsu_n,
                                                                                                                                                                 cct,
                                                                                                                                                                 cc_n,
                                                                                                                                                                 lfsvt,
                                                                                                                                                                 lfsv_n,
                                                                                                                                                                 avt,
                                                                                                                                                                 av_s,
                                                                                                                                                                 avt,
                                                                                                                                                                 av_n,
                                                                                                                                                                 )
            ]


@symbol
def jacobian_volume_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu = name_trialfunctionspace(Restriction.NONE)
    lfsvt = type_testfunctionspace()
    lfsv = name_testfunctionspace(Restriction.NONE)
    cct = type_coefficientcontainer()
    cc = name_coefficientcontainer(Restriction.NONE)
    avt = type_accumulation_variable()
    av = name_accumulation_variable((Restriction.NONE, Restriction.NONE))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void jacobian_volume(const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}) const'.format(geot,
                                                                                                                geo,
                                                                                                                lfsut,
                                                                                                                lfsu,
                                                                                                                cct,
                                                                                                                cc,
                                                                                                                lfsvt,
                                                                                                                lfsv,
                                                                                                                avt,
                                                                                                                av,
                                                                                                                )
            ]


@symbol
def jacobian_boundary_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu = name_trialfunctionspace(Restriction.NEGATIVE)
    lfsvt = type_testfunctionspace()
    lfsv = name_testfunctionspace(Restriction.NEGATIVE)
    cct = type_coefficientcontainer()
    cc = name_coefficientcontainer(Restriction.NEGATIVE)
    avt = type_accumulation_variable()
    av = name_accumulation_variable((Restriction.NEGATIVE, Restriction.NEGATIVE))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void jacobian_boundary(const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}) const'.format(geot,
                                                                                                                  geo,
                                                                                                                  lfsut,
                                                                                                                  lfsu,
                                                                                                                  cct,
                                                                                                                  cc,
                                                                                                                  lfsvt,
                                                                                                                  lfsv,
                                                                                                                  avt,
                                                                                                                  av,
                                                                                                                  )
            ]


@symbol
def jacobian_skeleton_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu_s = name_trialfunctionspace(Restriction.NEGATIVE)
    lfsu_n = name_trialfunctionspace(Restriction.POSITIVE)
    lfsvt = type_testfunctionspace()
    lfsv_s = name_testfunctionspace(Restriction.NEGATIVE)
    lfsv_n = name_testfunctionspace(Restriction.POSITIVE)
    cct = type_coefficientcontainer()
    cc_s = name_coefficientcontainer(Restriction.NEGATIVE)
    cc_n = name_coefficientcontainer(Restriction.POSITIVE)
    avt = type_accumulation_variable()
    av_ss = name_accumulation_variable((Restriction.NEGATIVE, Restriction.NEGATIVE))
    av_sn = name_accumulation_variable((Restriction.NEGATIVE, Restriction.POSITIVE))
    av_ns = name_accumulation_variable((Restriction.POSITIVE, Restriction.NEGATIVE))
    av_nn = name_accumulation_variable((Restriction.POSITIVE, Restriction.POSITIVE))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void jacobian_skeleton(const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}, {}& {}, {}& {}, {}& {}) const'.format(geot,
                                                                                                                                                                                    geo,
                                                                                                                                                                                    lfsut,
                                                                                                                                                                                    lfsu_s,
                                                                                                                                                                                    cct,
                                                                                                                                                                                    cc_s,
                                                                                                                                                                                    lfsvt,
                                                                                                                                                                                    lfsv_s,
                                                                                                                                                                                    lfsut,
                                                                                                                                                                                    lfsu_n,
                                                                                                                                                                                    cct,
                                                                                                                                                                                    cc_n,
                                                                                                                                                                                    lfsvt,
                                                                                                                                                                                    lfsv_n,
                                                                                                                                                                                    avt,
                                                                                                                                                                                    av_ss,
                                                                                                                                                                                    avt,
                                                                                                                                                                                    av_sn,
                                                                                                                                                                                    avt,
                                                                                                                                                                                    av_ns,
                                                                                                                                                                                    avt,
                                                                                                                                                                                    av_nn,
                                                                                                                                                                                    )
            ]


@symbol
def jacobian_apply_volume_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu = name_trialfunctionspace(Restriction.NONE)
    lfsvt = type_testfunctionspace()
    lfsv = name_testfunctionspace(Restriction.NONE)
    cct = type_coefficientcontainer()
    ac = name_applycontainer(Restriction.NONE)
    avt = type_accumulation_variable()
    av = name_accumulation_variable((Restriction.NONE,))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void jacobian_apply_volume(const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}) const'.format(
                geot,
                geo,
                lfsut,
                lfsu,
                cct,
                ac,
                lfsvt,
                lfsv,
                avt,
                av,)
            ]


@symbol
def jacobian_apply_boundary_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu = name_trialfunctionspace(Restriction.NEGATIVE)
    lfsvt = type_testfunctionspace()
    lfsv = name_testfunctionspace(Restriction.NEGATIVE)
    cct = type_coefficientcontainer()
    ac = name_applycontainer(Restriction.NEGATIVE)
    avt = type_accumulation_variable()
    av = name_accumulation_variable((Restriction.NEGATIVE,))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void jacobian_apply_boundary(const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}) const'.format(
                geot,
                geo,
                lfsut,
                lfsu,
                cct,
                ac,
                lfsvt,
                lfsv,
                avt,
                av,)
            ]


@symbol
def jacobian_apply_skeleton_signature():
    geot = type_geometry_wrapper()
    geo = name_geometry_wrapper()
    lfsut = type_trialfunctionspace()
    lfsu_s = name_trialfunctionspace(Restriction.NEGATIVE)
    lfsu_n = name_trialfunctionspace(Restriction.POSITIVE)
    lfsvt = type_testfunctionspace()
    lfsv_s = name_testfunctionspace(Restriction.NEGATIVE)
    lfsv_n = name_testfunctionspace(Restriction.POSITIVE)
    cct = type_coefficientcontainer()
    ac_s = name_applycontainer(Restriction.NEGATIVE)
    ac_n = name_applycontainer(Restriction.POSITIVE)
    avt = type_accumulation_variable()
    av_s = name_accumulation_variable((Restriction.NEGATIVE,))
    av_n = name_accumulation_variable((Restriction.POSITIVE,))
    return ['template<typename {}, typename {}, typename {}, typename {}, typename {}>'.format(geot,
                                                                                               lfsut,
                                                                                               cct,
                                                                                               lfsvt,
                                                                                               avt,
                                                                                               ),
            'void jacobian_apply_skeleton(const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, const {}& {}, {}& {}, {}& {}) const'.format(
                geot,
                geo,
                lfsut,
                lfsu_s,
                cct,
                ac_s,
                lfsvt,
                lfsv_s,
                lfsut,
                lfsu_n,
                cct,
                ac_n,
                lfsvt,
                lfsv_n,
                avt,
                av_s,
                avt,
                av_n,)
            ]
